﻿/*
 * (c) Copyright Ascensio System SIA 2010-2017
 *
 * This program is a free software product. You can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License (AGPL)
 * version 3 as published by the Free Software Foundation. In accordance with
 * Section 7(a) of the GNU AGPL its Section 15 shall be amended to the effect
 * that Ascensio System SIA expressly excludes the warranty of non-infringement
 * of any third-party rights.
 *
 * This program is distributed WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE. For
 * details, see the GNU AGPL at: http://www.gnu.org/licenses/agpl-3.0.html
 *
 * You can contact Ascensio System SIA at Lubanas st. 125a-25, Riga, Latvia,
 * EU, LV-1021.
 *
 * The  interactive user interfaces in modified source and object code versions
 * of the Program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU AGPL version 3.
 *
 * Pursuant to Section 7(b) of the License you must retain the original Product
 * logo when distributing the program. Pursuant to Section 7(e) we decline to
 * grant you any rights under trademark law for use of our trademarks.
 *
 * All the Product's GUI elements, including illustrations and icon sets, as
 * well as technical writing content are licensed under the terms of the
 * Creative Commons Attribution-ShareAlike 4.0 International. See the License
 * terms at http://creativecommons.org/licenses/by-sa/4.0/legalcode
 *
 */
#pragma once
#ifndef PPTX_LOGIC_BUAUTONUM_INCLUDE_H_
#define PPTX_LOGIC_BUAUTONUM_INCLUDE_H_

#include "./../../WrapperWritingElement.h"
#include "./../../Limit/TextAutonumberScheme.h"

namespace PPTX
{
	namespace Logic
	{
		class BuAutoNum : public WrapperWritingElement
		{
		public:
			WritingElement_AdditionConstructors(BuAutoNum)
			PPTX_LOGIC_BASE2(BuAutoNum)

			BuAutoNum& operator=(const BuAutoNum& oSrc)
			{
				parentFile		= oSrc.parentFile;
				parentElement	= oSrc.parentElement;

				type	= oSrc.type;
				startAt = oSrc.startAt;

				return *this;
			}
			virtual void fromXML(XmlUtils::CXmlLiteReader& oReader)
			{
				ReadAttributes( oReader );
			}
			virtual OOX::EElementType getType() const
			{
				return OOX::et_a_buChar;
			}
			virtual void fromXML(XmlUtils::CXmlNode& node)
			{
				node.ReadAttributeBase(L"type", type);
				node.ReadAttributeBase(L"startAt", startAt);

				Normalize();
			}
			void ReadAttributes(XmlUtils::CXmlLiteReader& oReader)
			{
				WritingElement_ReadAttributes_Start	( oReader )
					WritingElement_ReadAttributes_Read_if	  ( oReader, L"startAt",	startAt)
					WritingElement_ReadAttributes_Read_else_if( oReader, L"type",		type)
				WritingElement_ReadAttributes_End	( oReader )		
				
				Normalize();
			}
			virtual std::wstring toXML() const
			{
				XmlUtils::CAttribute oAttr;
				oAttr.Write(_T("type"), type.get());
				oAttr.Write(_T("startAt"), startAt);

				return XmlUtils::CreateNode(_T("a:buAutoNum"), oAttr);
			}
			virtual void toXmlWriter(NSBinPptxRW::CXmlWriter* pWriter) const
			{
				pWriter->StartNode(_T("a:buAutoNum"));
				pWriter->StartAttributes();
				pWriter->WriteAttribute(_T("type"), type.get());
				pWriter->WriteAttribute(_T("startAt"), startAt);
				pWriter->EndAttributes();
				pWriter->EndNode(_T("a:buAutoNum"));
			}

			virtual void toPPTY(NSBinPptxRW::CBinaryFileWriter* pWriter) const
			{
				pWriter->StartRecord(BULLET_TYPE_BULLET_AUTONUM);

				pWriter->WriteBYTE(NSBinPptxRW::g_nodeAttributeStart);				
				pWriter->WriteLimit1(0, type);
				pWriter->WriteInt2(1, startAt);
				pWriter->WriteBYTE(NSBinPptxRW::g_nodeAttributeEnd);

				pWriter->EndRecord();
			}
		
		public:
			Limit::TextAutonumberScheme type;
			nullable_int				startAt;

		public:
			AVSINLINE void Normalize()
			{
				startAt.normalize(1, 32767);
			}

		protected:
			virtual void FillParentPointersForChilds(){};
		};
	} // namespace Logic
} // namespace PPTX

#endif // PPTX_LOGIC_BUAUTONUM_INCLUDE_H