﻿/*
 * (c) Copyright Ascensio System SIA 2010-2017
 *
 * This program is a free software product. You can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License (AGPL)
 * version 3 as published by the Free Software Foundation. In accordance with
 * Section 7(a) of the GNU AGPL its Section 15 shall be amended to the effect
 * that Ascensio System SIA expressly excludes the warranty of non-infringement
 * of any third-party rights.
 *
 * This program is distributed WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE. For
 * details, see the GNU AGPL at: http://www.gnu.org/licenses/agpl-3.0.html
 *
 * You can contact Ascensio System SIA at Lubanas st. 125a-25, Riga, Latvia,
 * EU, LV-1021.
 *
 * The  interactive user interfaces in modified source and object code versions
 * of the Program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU AGPL version 3.
 *
 * Pursuant to Section 7(b) of the License you must retain the original Product
 * logo when distributing the program. Pursuant to Section 7(e) we decline to
 * grant you any rights under trademark law for use of our trademarks.
 *
 * All the Product's GUI elements, including illustrations and icon sets, as
 * well as technical writing content are licensed under the terms of the
 * Creative Commons Attribution-ShareAlike 4.0 International. See the License
 * terms at http://creativecommons.org/licenses/by-sa/4.0/legalcode
 *
 */
#pragma once
#ifndef PPTX_LOGIC_COLORMODIFIER_INCLUDE_H_
#define PPTX_LOGIC_COLORMODIFIER_INCLUDE_H_

#include "./../../WrapperWritingElement.h"

namespace PPTX
{
	namespace Logic
	{
		class ColorModifier : public WrapperWritingElement
		{
		public:
			WritingElement_AdditionConstructors(ColorModifier)
			PPTX_LOGIC_BASE2(ColorModifier)

			virtual OOX::EElementType getType() const
			{
				return OOX::et_a_prstClr;
			}	
			void fromXML(XmlUtils::CXmlLiteReader& oReader)
			{
				name = oReader.GetName();

				ReadAttributes( oReader );

				if (XmlUtils::GetNameNoNS(name) == _T("alpha"))
				{
					ReadAttributes2( oReader );
				}
			}
			void ReadAttributes2(XmlUtils::CXmlLiteReader& oReader)
			{
				nullable_string sTmp;
				WritingElement_ReadAttributes_Start( oReader )
					WritingElement_ReadAttributes_ReadSingle ( oReader, _T("val"), sTmp)
				WritingElement_ReadAttributes_End( oReader )

                if (val.is_init() && sTmp.is_init() && sTmp->find(wchar_t('%')) != -1)
					*val = (*val) * 1000; 
			}
			void ReadAttributes(XmlUtils::CXmlLiteReader& oReader)
			{
				WritingElement_ReadAttributes_Start( oReader )
					WritingElement_ReadAttributes_ReadSingle ( oReader, _T("val"), val)
				WritingElement_ReadAttributes_End( oReader )
			}
			virtual void fromXML(XmlUtils::CXmlNode& node)
			{
				name	= node.GetName();
				node.ReadAttributeBase(L"val", val);

				if (XmlUtils::GetNameNoNS(name) == _T("alpha"))
				{
					nullable_string sTmp;
					node.ReadAttributeBase(L"val", sTmp);

                    if (val.is_init() && sTmp.is_init() && sTmp->find(wchar_t('%')) != -1)
						*val = (*val) * 1000;
				}
			}
			virtual std::wstring toXML() const
			{
				XmlUtils::CAttribute oAttr;
				oAttr.Write(_T("val"), val);
				return XmlUtils::CreateNode(name, oAttr);
			}

			ColorModifier& operator=(const ColorModifier& oSrc)
			{
				parentFile		= oSrc.parentFile;
				parentElement	= oSrc.parentElement;

				name		= oSrc.name;
				val			= oSrc.val;
				return *this;
			}

			virtual void toPPTY(NSBinPptxRW::CBinaryFileWriter* pWriter) const
			{
				pWriter->WriteBYTE(NSBinPptxRW::g_nodeAttributeStart);
				pWriter->WriteString1(0, name);
				pWriter->WriteInt2(1, val);
				pWriter->WriteBYTE(NSBinPptxRW::g_nodeAttributeEnd);
			}

			virtual void toXmlWriter(NSBinPptxRW::CXmlWriter* pWriter) const
			{				
				std::wstring _name;
				std::wstring sAttrNamespace;
				if (XMLWRITER_DOC_TYPE_WORDART == pWriter->m_lDocType)
				{
					_name = _T("w14:") + XmlUtils::GetNameNoNS(name);
					sAttrNamespace = _T("w14:");
				}
				else
					_name = name;
				pWriter->StartNode(_name);
				pWriter->StartAttributes();
				pWriter->WriteAttribute(sAttrNamespace + _T("val"), val);
				pWriter->EndAttributes();
				pWriter->EndNode(_name);
			}

		public:
			std::wstring			name;
			nullable_int	val;
		protected:
			virtual void FillParentPointersForChilds(){};
		};
	} // namespace Logic
} // namespace PPTX

#endif // PPTX_LOGIC_COLORMODIFIER_INCLUDE_H
